using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class MultipleMeshes : MonoBehaviour
{
    List<MeshRenderer> children = new List<MeshRenderer>();
    public GameObject model;
    public bool hideModel = true;
    int maxParticles;
    double totalSurfaceArea;
    public float delay = 0.1f;
    List<GameObject> particleSystems = new List<GameObject>();
    void OnEnable()
    {
        Invoke("CreateParticles", delay);
    }
    void CreateParticles()
    {
        particleSystems.Clear();
        totalSurfaceArea = 0d;
        maxParticles = GetComponent<ParticleSystem>().main.maxParticles;
        children.AddRange(model.GetComponentsInChildren<MeshRenderer>());
        List<MeshFilter> meshFilters = new List<MeshFilter>();
        meshFilters.AddRange(model.GetComponentsInChildren<MeshFilter>());
        foreach (MeshFilter f in meshFilters)
        {
            List<int> triangles = new List<int>();
            triangles.AddRange(f.mesh.triangles);
            List<Vector3> vertices = new List<Vector3>();
            vertices.AddRange(f.mesh.vertices);
            for (int i = 0; i < triangles.Count; i += 3)
            {
                Vector3 corner = vertices[triangles[i]];
                Vector3 a = vertices[triangles[i + 1]] - corner;
                Vector3 b = vertices[triangles[i + 2]] - corner;
                totalSurfaceArea += (Vector3.Cross(a, b).magnitude / 2d);
            }
        }
        foreach (MeshRenderer renderer in children)
        {
            GameObject child = Instantiate(gameObject);
            foreach(Transform g in child.transform)
            {
                Destroy(g.gameObject);
            }
            Destroy(child.GetComponent<MultipleMeshes>());
            if(child.GetComponent<SaveEffect>() != null)
            {
                Destroy(child.GetComponent<SaveEffect>());
            }
            if (child.GetComponent<Morphing>() != null)
            {
                Destroy(child.GetComponent<Morphing>());
            }
            particleSystems.Add(child);


            var shape = child.GetComponent<ParticleSystem>().shape;
            shape.meshRenderer = renderer;
            shape.meshShapeType = GetComponent<ParticleSystem>().shape.meshShapeType;

            List<int> triangles = new List<int>();
            triangles.AddRange(renderer.gameObject.GetComponent<MeshFilter>().mesh.triangles);
            List<Vector3> vertices = new List<Vector3>();
            vertices.AddRange(renderer.gameObject.GetComponent<MeshFilter>().mesh.vertices);
            double surfaceArea = 0;
            for (int i = 0; i < triangles.Count; i += 3)
            {
                Vector3 corner = vertices[triangles[i]];
                Vector3 a = vertices[triangles[i + 1]] - corner;
                Vector3 b = vertices[triangles[i + 2]] - corner;
                surfaceArea += (Vector3.Cross(a, b).magnitude / 2d);
            }
            var emission = child.GetComponent<ParticleSystem>().emission;
            if(emission.rateOverTime.mode == ParticleSystemCurveMode.Constant)
            {
                double rateOverTime = emission.rateOverTime.constant * (surfaceArea / totalSurfaceArea);
                emission.rateOverTime = (float) rateOverTime;
            }
            if (emission.rateOverTime.mode == ParticleSystemCurveMode.TwoConstants)
            {
                double min = emission.rateOverTime.constantMin * (surfaceArea / totalSurfaceArea);
                double max = emission.rateOverTime.constantMax * (surfaceArea / totalSurfaceArea);
                var rateOverTime = emission.rateOverTime;
                rateOverTime.constantMin = (float) min;
                rateOverTime.constantMax = (float) max;
                emission.rateOverTime = rateOverTime;
            }
            if (emission.rateOverTime.mode == ParticleSystemCurveMode.Curve)
            {
                double rateOverTime = emission.rateOverTime.curveMultiplier * (surfaceArea / totalSurfaceArea);
                emission.rateOverTime = new ParticleSystem.MinMaxCurve((float) rateOverTime, emission.rateOverTime.curve);
            }
            var main = child.GetComponent<ParticleSystem>().main;
            double maxParticles = main.maxParticles * surfaceArea / totalSurfaceArea;
            main.maxParticles = (int) maxParticles;
            if(hideModel == true)
            {
                renderer.enabled = false;
            }
        }
        ParticleSystem.Particle[] p = new ParticleSystem.Particle[0];
        ParticleSystem original = GetComponent<ParticleSystem>();
        original.SetParticles(p);
        original.Stop(false);
        var originalMain = original.main;
        originalMain.maxParticles = 0;
        foreach (GameObject g in particleSystems)
        {
            g.transform.SetParent(transform);
        }
    }
}
